--
-- OilPallet
-- Specialization for Oil change
--
-- @author  	JoXXer / 
-- @version 	v1.0
-- @date  		16/11/11 / 
-- @history:	v1.0 - Initial version / 
--

OilPallet = {};

function OilPallet.prerequisitesPresent(specializations)
    return true;
end;

function OilPallet:load(xmlFile)

	self.changeOilPallet = SpecializationUtil.callSpecializationsFunction("changeOilPallet");

	self.checkString = Utils.getNoNil(getXMLString(xmlFile, "vehicle.oilPallet#checkString"), "changeOil")
	local oilCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.oilPallet#capacity"),60);
	self.oilCapacity = oilCapacity;
	self.oilDefaultCapacity = oilCapacity;
	local cans = getXMLString(xmlFile, "vehicle.oilPallet#cans");
	self.cans = {};
	local canObject = nil
	if cans ~= nil then
		canObject = Utils.indexToObject(self.components, cans);
		if canObject ~= nil then
			local num = getNumOfChildren(canObject);
			for i=0, num-1 do
				local child = getChildAt(canObject, i);
				if child ~= nil then
					table.insert(self.cans, child);
				end;
			end;
		end;
	end;
	self.canCount = table.getn(self.cans);
	self.lastCapacity = 0;

	self.oilPalletVehicle = nil;
end;

function OilPallet:delete()
end;

function OilPallet:readStream(streamId, connection)
	self.oilCapacity = streamReadFloat32(streamId);
end;

function OilPallet:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self.oilCapacity);
end;

function OilPallet:mouseEvent(posX, posY, isDown, isUp, button)
end;

function OilPallet:keyEvent(unicode, sym, modifier, isDown)
end;

function OilPallet:update(dt)
end;

function OilPallet:updateTick(dt)
	if self.lastCapacity ~= self.oilCapacity then
		if self.canCount ~= nil and self.canCount > 0 then
			local oilPerCan = self.oilDefaultCapacity / self.canCount;
			local oilLeft = oilPerCan;
			local capCheck = self.oilCapacity + oilPerCan;
			for i=1, table.getn(self.cans) do
				if capCheck > oilLeft then
					if not getVisibility(self.cans[i]) then
						setVisibility(self.cans[i], true);
					end;
				else
					if getVisibility(self.cans[i]) then
						setVisibility(self.cans[i], false);
					end;
				end;
				oilLeft = oilLeft + oilPerCan;
			end;
		end;
		self.lastCapacity = self.oilCapacity;
	end;
	local px, py, pz = getWorldTranslation(self.rootNode);
	local vehicle = g_currentMission.controlledVehicle;
	if vehicle ~= nil and vehicle.oilChange ~= nil and vehicle.oilChange.checkString == self.checkString then
		local vx, vy, vz = getWorldTranslation(vehicle.rootNode);
		local distance = Utils.vector2Length(px-vx, pz-vz);
		if distance < 10 then
			if vehicle.oilChange.oilInRange == nil or (vehicle.oilChange.oilInRange ~= nil and vehicle.oilChange.oilInRange ~= self and vehicle.neededOilPerChange <= self.oilCapacity and self.oilCapacity < vehicle.oilChange.oilInRange.oilCapacity) then
				vehicle.oilChange.oilInRange = self;
			end;
		else
			if vehicle.oilChange.oilInRange == self then
				vehicle.oilChange.oilInRange = nil;
			end;
		end;
	end;
end;

function OilPallet:draw()
end;

function OilPallet:changeOilPallet(vehicle, noEventSend)
	if vehicle ~= nil and vehicle.oilChange ~= nil then
		OilChangeEventGlobal.sendEvent(self, vehicle, noEventSend);
		vehicle:changeOil(self);
	end;
end;

function OilPallet:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local valueInt = getXMLFloat(xmlFile, key.."#oilCapacity");
	if valueInt ~= nil then
		self.oilCapacity = valueInt;
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function OilPallet:getSaveAttributesAndNodes(nodeIdent)
	local attributes = nil;
	attributes = 'oilCapacity="'.. self.oilCapacity .. '"';
	return attributes, nil;
end;


OilChangeEventGlobal = {};
OilChangeEventGlobal_mt = Class(OilChangeEventGlobal, Event);

InitEventClass(OilChangeEventGlobal, "OilChangeEventGlobal");

function OilChangeEventGlobal:emptyNew()
	local self = Event:new(OilChangeEventGlobal_mt);
	self.className="OilChangeEventGlobal";
	return self;
end;

function OilChangeEventGlobal:new(vehicle, oilChangeVehicle)
	local self = OilChangeEventGlobal:emptyNew();
	self.vehicle = vehicle;
	self.oilChangeVehicle = oilChangeVehicle;
	return self;
end;

function OilChangeEventGlobal:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	local attacherId = streamReadInt32(streamId);
	self.oilChangeVehicle = networkGetObject(attacherId);
	self.vehicle = networkGetObject(id);
	self:run(connection);
end;

function OilChangeEventGlobal:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteInt32(streamId, networkGetObjectId(self.oilChangeVehicle));
end;

function OilChangeEventGlobal:run(connection)
	if self.vehicle ~= nil then
		self.vehicle:changeOilPallet(self.oilChangeVehicle, true);
		if not connection:getIsServer() then
			g_server:broadcastEvent(OilChangeEventGlobal:new(self.vehicle, self.oilChangeVehicle), nil, connection, self.vehicle);
		end;
	end;
end;


function OilChangeEventGlobal.sendEvent(vehicle, oilChangeVehicle, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(OilChangeEventGlobal:new(vehicle, oilChangeVehicle), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(OilChangeEventGlobal:new(vehicle, oilChangeVehicle));
		end;
	end;
end;
